@@ -32,11 +32,14 @@
 #include <glm/gtc/matrix_transform.hpp>
 #include <glm/gtc/type_ptr.hpp>
 #include "Renderer/ShaderEngine.hpp"
+#include "Renderer/StaticGlShaders.h"
 
 
 void projectMSDL::audioInputCallbackF32(void *userdata, unsigned char *stream, int len) {
     projectMSDL *app = (projectMSDL *) userdata;
-    //    printf("LEN: %i\n", len);
+//    printf("\nLEN: %i\n", len);
+//    for (int i = 0; i < 64; i++)
+//        printf("%X ", stream[i]);
     // stream is (i think) samples*channels floats (native byte order) of len BYTES
     if (app->audioChannelsCount == 1)
         app->pcm()->addPCMfloat((float *)stream, len/sizeof(float));
@@ -152,7 +155,6 @@ int projectMSDL::openAudioInput() {
 #endif
     
     // get audio input device
-    unsigned int i;
     NumAudioDevices = SDL_GetNumAudioDevices(true);  // capture, please
 
     CurAudioDevice = 0;
@@ -163,7 +165,7 @@ int projectMSDL::openAudioInput() {
         return 0;
     }
 #ifdef DEBUG
-    for (i = 0; i < NumAudioDevices; i++) {
+    for (unsigned int i = 0; i < NumAudioDevices; i++) {
         SDL_Log("Found audio capture device %d: %s", i, SDL_GetAudioDeviceName(i, true));
     }
 #endif
@@ -249,7 +251,7 @@ void projectMSDL::nextMonitor()
 		std::vector<SDL_Rect> displayBounds;
 		int nextWindow = currentWindowIndex + 1;
 		if (nextWindow >= displayCount) nextWindow = 0;
-
+        
 		for (int i = 0; i < displayCount; i++)
 		{
 			displayBounds.push_back(SDL_Rect());
@@ -273,6 +275,17 @@ void projectMSDL::toggleFullScreen() {
     }
 }
 
+void projectMSDL::scrollHandler(SDL_Event* sdl_evt) {
+     // handle mouse scroll wheel - up++
+    if (sdl_evt->wheel.y > 0) {
+        projectM::selectPrevious(true);
+    }
+    // handle mouse scroll wheel - down--
+    if (sdl_evt->wheel.y < 0) {
+        projectM::selectNext(true);
+    }
+}
+
 void projectMSDL::keyHandler(SDL_Event *sdl_evt) {
     projectMEvent evt;
     projectMKeycode key;
@@ -280,119 +293,123 @@ void projectMSDL::keyHandler(SDL_Event *sdl_evt) {
     SDL_Keymod sdl_mod = (SDL_Keymod) sdl_evt->key.keysym.mod;
     SDL_Keycode sdl_keycode = sdl_evt->key.keysym.sym;
 
+    // Left or Right Gui or Left Ctrl
+    if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
+        keymod = true;
+
 	// handle keyboard input (for our app first, then projectM)
     switch (sdl_keycode) {
-        case SDLK_q:
-            if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL) {
-                // cmd/ctrl-q = quit
-                done = 1;
-                return;
+
+    case SDLK_q:
+        if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL) {
+            // cmd/ctrl-q = quit
+            done = 1;
+            return;
+        }
+        break;
+    case SDLK_BACKSPACE:
+        projectM::deleteSearchText();
+        break;
+    case SDLK_SLASH:
+        break;
+    case SDLK_BACKSLASH:
+        break;
+    case SDLK_RETURN:
+        if (!projectM::isTextInputActive()) {
+            SDL_StartTextInput();
+        }
+        break;
+    case SDLK_ESCAPE:
+        if (projectM::isTextInputActive())                
+            SDL_StopTextInput();
+        break;
+    case SDLK_i:
+        if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
+        {
+            toggleAudioInput();
+            return; // handled
+        }
+        break;
+    case SDLK_s:
+        if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
+        {
+            // command-s: [s]tretch monitors
+            // Stereo requires fullscreen
+#if !STEREOSCOPIC_SBS
+            if (!this->stretch) { // if stretching is not already enabled, enable it.
+                stretchMonitors();
+                this->stretch = true;
+            } else {
+                toggleFullScreen(); // else, just toggle full screen so we leave stretch mode.
+                this->stretch = false;
             }
-            break;
-        case SDLK_i:
-                if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
-                {
-                    toggleAudioInput();
-                    return; // handled
-                }
-            break;
-		case SDLK_s:
-			if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
-			{
-				// command-s: [s]tretch monitors
-				// Stereo requires fullscreen
-#if !STEREOSCOPIC_SBS
-				stretchMonitors();
 #endif
-				return; // handled
-			}
-		case SDLK_m:
-			if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
-			{
-				// command-m: change [m]onitor
-				// Stereo requires fullscreen
+            return; // handled
+        }
+    case SDLK_m:
+        if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL)
+        {
+            // command-m: change [m]onitor
+            // Stereo requires fullscreen
 #if !STEREOSCOPIC_SBS
-				nextMonitor();
+            nextMonitor();
 #endif
-				return; // handled
-			}
-        case SDLK_f:
-            if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL) {
-                // command-f: fullscreen
-				// Stereo requires fullscreen
+            this->stretch = false; // if we are switching monitors, ensure we disable monitor stretching.
+            return; // handled
+        }
+    case SDLK_f:
+        if (sdl_mod & KMOD_LGUI || sdl_mod & KMOD_RGUI || sdl_mod & KMOD_LCTRL) {
+            // command-f: fullscreen
+            // Stereo requires fullscreen
 #if !STEREOSCOPIC_SBS
-				toggleFullScreen();
+            toggleFullScreen();
 #endif
-                return; // handled
+            this->stretch = false; // if we are toggling fullscreen, ensure we disable monitor stretching.
+            return; // handled
+        }
+        break;
+    case SDLK_LEFT:
+        // selectPrevious(true);
+        break;
+    case SDLK_RIGHT:
+        // selectNext(true);
+        break;
+    case SDLK_UP:
+        break;
+    case SDLK_DOWN:
+        break;
+
+    case SDLK_F3:
+        break;
+
+
+    case SDLK_SPACE:
+        if (!projectM::isTextInputActive(true))
+            setPresetLock(!isPresetLocked());
+        break;
+    case SDLK_F1:
+        break;
+    case SDLK_DELETE:
+        /*
+        try {
+            if (selectedPresetIndex(index)) {
+                DeleteFile(
+                    LPCSTR(
+                        getPresetURL(index).c_str()
+                    )
+                );
             }
-            break;
-					case SDLK_LEFT:
-						// selectPrevious(true);
-						break;
-					case SDLK_RIGHT:
-						// selectNext(true);
-						break;
-					case SDLK_UP:
-						break;
-					case SDLK_DOWN:
-						break;
-
-					case SDLK_F3:
-						break;
-
-
-					case SDLK_SPACE:
-						setPresetLock(
-							!isPresetLocked()
-						);
-						break;
-					case SDLK_F1:
-					case SDLK_ESCAPE:
-
-						// exit(1);
-						// show help with other keys
-						sdl_keycode = SDLK_F1;
-						break;
-					case SDLK_DELETE:
-						/*
-						try {
-							if (selectedPresetIndex(index)) {
-								DeleteFile(
-									LPCSTR(
-										getPresetURL(index).c_str()
-									)
-								);
-							}
-						}
-						catch (const std::exception & e) {
-							printf("Delete failed");
-						}
-						*/
-						break;
-					case SDLK_RETURN:
-						/*
-						try {
-							if (selectedPresetIndex(index)) {
-								CopyFile(
-										LPCSTR(
-												app->getPresetURL(index).c_str()
-										),
-										LPCTSTR(L"C:\\"),
-										false
-								);
-							}
-						}
-						catch (const std::exception & e) {
-							printf("Delete failed");
-						}
-								*/
-						break;
+        }
+        catch (const std::exception & e) {
+            printf("Delete failed");
+        }
+        */
+        break;
     }
-
     // translate into projectM codes and perform default projectM handler
     evt = sdl2pmEvent(sdl_evt);
-    key = sdl2pmKeycode(sdl_keycode);
     mod = sdl2pmModifier(sdl_mod);
+    key = sdl2pmKeycode(sdl_keycode,sdl_mod);
     key_handler(evt, key, mod);
 }
 
@@ -426,7 +443,12 @@ void projectMSDL::resize(unsigned int width_, unsigned int height_) {
 
 void projectMSDL::pollEvent() {
     SDL_Event evt;
-
+    
+    int mousex = 0;
+    float mousexscale = 0;
+    int mousey = 0;
+    float mouseyscale = 0;
+    int mousepressure = 0;
     while (SDL_PollEvent(&evt))
     {
         switch (evt.type) {
@@ -442,14 +464,95 @@ void projectMSDL::pollEvent() {
 						break;
                 }
                 break;
+            case SDL_MOUSEWHEEL:
+                scrollHandler(&evt);
             case SDL_KEYDOWN:
                 keyHandler(&evt);
                 break;
+            case SDL_MOUSEBUTTONDOWN:
+                if (evt.button.button == SDL_BUTTON_LEFT) {
+                    // if it's the first mouse down event (since mouse up or since SDL was launched)
+                    if (!mouseDown) {
+                        // Get mouse coorindates when you click.
+                        SDL_GetMouseState(&mousex, &mousey);
+                        // Scale those coordinates. libProjectM supports a scale of 0.1 instead of absolute pixel coordinates.
+                        mousexscale = (mousex / (float)width);
+                        mouseyscale = ((height - mousey) / (float)height);
+                        // Touch. By not supplying a touch type, we will default to random.
+                        touch(mousexscale, mouseyscale, mousepressure);
+                        mouseDown = true;
+                    }
+                }
+                else if (evt.button.button == SDL_BUTTON_RIGHT)
+                {
+                    mouseDown = false;
+
+                    // Keymod = Left or Right Gui or Left Ctrl. This is a shortcut to remove all waveforms.
+                    if (keymod) {
+                        touchDestroyAll();
+                        keymod = false;
+                        break;
+                    }
+
+                    // Right Click
+                    SDL_GetMouseState(&mousex, &mousey);
+
+                    // Scale those coordinates. libProjectM supports a scale of 0.1 instead of absolute pixel coordinates.
+                    mousexscale = (mousex / (float)width);
+                    mouseyscale = ((height - mousey) / (float)height);
+
+                    // Destroy at the coordinates we clicked.
+                    touchDestroy(mousexscale, mouseyscale);
+                }
+                break;
+            case SDL_MOUSEBUTTONUP:
+                mouseDown = false;
+                break;
+            case SDL_TEXTINPUT:
+                if (projectM::isTextInputActive(true))
+                {
+                    projectM::setSearchText(evt.text.text);
+                    projectM::populatePresetMenu();
+                }
+                break;
             case SDL_QUIT:
                 done = true;
                 break;
         }
     }
+
+    // Handle dragging your waveform when mouse is down.
+    if (mouseDown) {
+        // Get mouse coordinates when you click.
+        SDL_GetMouseState(&mousex, &mousey);
+        // Scale those coordinates. libProjectM supports a scale of 0.1 instead of absolute pixel coordinates.
+        mousexscale = (mousex / (float)width);
+        mouseyscale = ((height - mousey) / (float)height);
+        // Drag Touch. 
+        touchDrag(mousexscale, mouseyscale, mousepressure);
+    }
+}
+
+// This touches the screen to generate a waveform at X / Y. 
+void projectMSDL::touch(float x, float y, int pressure, int touchtype) {
+#ifdef PROJECTM_TOUCH_ENABLED
+    projectM::touch(x, y, pressure, touchtype);
+#endif
+}
+
+// This moves the X Y of your existing waveform that was generated by a touch (only if you held down your click and dragged your mouse around).
+void projectMSDL::touchDrag(float x, float y, int pressure) {
+    projectM::touchDrag(x, y, pressure);
+}
+
+// Remove waveform at X Y
+void projectMSDL::touchDestroy(float x, float y) {
+    projectM::touchDestroy(x, y);
+}
+
+// Remove all waveforms
+void projectMSDL::touchDestroyAll() {
+    projectM::touchDestroyAll();
 }
 
 void projectMSDL::renderFrame() {
@@ -491,7 +594,9 @@ void projectMSDL::init(SDL_Window *window, SDL_GLContext *_glCtx, const bool _re
     // are we rendering to a texture?
     renderToTexture = _renderToTexture;
     if (renderToTexture) {
-        programID = ShaderEngine::CompileShaderProgram(ShaderEngine::v2f_c4f_t2f_vert, ShaderEngine::v2f_c4f_t2f_frag, "v2f_c4f_t2f");
+        programID = ShaderEngine::CompileShaderProgram(
+            StaticGlShaders::Get()->GetV2fC4fT2fVertexShader(),
+            StaticGlShaders::Get()->GetV2fC4fT2fFragmentShader(), "v2f_c4f_t2f");
         textureID = projectM::initRenderToTexture();
 
         float points[16] = {
