@@ -107,7 +107,7 @@ std::string getConfigFilePath(std::string datadir_path) {
     SDL_LogWarn(SDL_LOG_CATEGORY_APPLICATION, "Cannot create ~/.projectM/config.inp, using default config file\n");
     return std::string(projectM_config);
   } else {
-    SDL_LogWarn(SDL_LOG_CATEGORY_ERROR, "Using implementation defaults, your system is really messed up, I'm suprised we even got this far\n");
+    SDL_LogWarn(SDL_LOG_CATEGORY_ERROR, "Using implementation defaults, your system is really messed up, I'm surprised we even got this far\n");
 	return "";
   }
 }
@@ -304,8 +304,8 @@ srand((int)(time(NULL)));
 
 #else
 	// Disabling compatibility profile
-    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
-    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 2);
+    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
+    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 1);
     SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
 
 #endif
@@ -427,14 +427,16 @@ modKey = "CMD";
 		"N: Next preset""\n"
 		"P: Previous preset""\n"
 		"UP: Increase Beat Sensitivity""\n"
-		"DOWN: Decrease Beat Sensitivity""\n" +
-		modKey + "-I: Audio Input (listen to next device)""\n" +
-		modKey + "-M: Change Monitor""\n" +
-		modKey + "-S: Stretch Monitors""\n" +
-		modKey + "-F: Fullscreen""\n" +
-		modKey + "-Q: Quit";
+		"DOWN: Decrease Beat Sensitivity""\n"
+		"Left Click: Drop Random Waveform on Screen""\n"
+		"Right Click: Remove Random Waveform""\n" +
+		modKey + "+Right Click: Remove All Random Waveforms""\n" +
+		modKey + "+I: Audio Input (listen to next device)""\n" +
+		modKey + "+M: Change Monitor""\n" +
+		modKey + "+S: Stretch Monitors""\n" +
+		modKey + "+F: Fullscreen""\n" +
+		modKey + "+Q: Quit";
     app->setHelpText(sdlHelpMenu.c_str());
-    
     app->init(win, &glCtx);
 
 #if STEREOSCOPIC_SBS
